/* Centaur Makefile Generator
 *
 * Version 2.0
 * 11/19/2020
 *
 * Constants and function declarations for makegen.c
 *
 * makegen.h
 */

#ifndef _MAKEGEN_H
#define _MAKEGEN_H 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* Uncomment the line below to also write output to the screen. */
/* #define SHOW_OUTPUT 1*/

#define MAXLINELEN 256
#define MAXFILELEN (MAXLINELEN * 4)

#define MKTMPSTR "ls --ignore=.h *."

#define _MAKEFILE "CC=gcc\nCFLAGS=-I.\nDEPS=%s\n\n%%.o: %%.c $(DEPS)\n\t$(CC) -c -o $@ $< $(CFLAGS)\n\n%s: %s\n\t$(CC) %s -o %s %s\n\ninstall:\n\tmv -v %s %s\n\nclean:\n\trm -v *.o\n\nuninstall:\n\trm -i %s/%s\n"
#define _MAKEFILE_NOLIBS "CC=gcc\nCFLAGS=-I.\nDEPS=%s\n\n%%.o: %%.c $(DEPS)\n\t$(CC) -c -o $@ $< $(CFLAGS)\n\n%s: %s\n\t$(CC) -o %s %s\n\ninstall:\n\tmv -v %s %s\n\nclean:\n\trm -v *.o\n\nuninstall:\n\trm -i %s/%s\n"

#define MAKEINFO_FILE ".makeinfo"

#define HTEMPFILE "._makegen_h.tmp"
#define CTEMPFILE "._makegen_c.tmp"
#define OTEMPFILE "._makegen_o.tmp"

#define HELP_TEXT "makegen [-o prog-name] [-L 'library-args'] [-d install-dir]\n\nFiles in current directory with extension \".h\" and \".c\" are populated automatically.\n\n\tAll parameters are optional, but must be given in the above order\n\t(i.e. -o, -L, -d) when used.\n\tprog-name defaults to \"main\" when \"-o\" is not used.\n\tlibrary-args defaults to null when \"-L\" is not used.\n\tlibrary-args should be given as they would be for gcc (e.g. -lcrpyto).\n\tEnclose in single-quotes ('') as shown above if using space characters in library-args.\n\tinstall-dir defaults to \"/usr/bin\" when \"-d\" is not used.\n\nIf optional file '.makeinfo' is found in the current directory, the first line is used to set the program's full name and the second is used\nto set the version number. These values are used to generate the comment header to be prepended to the Makefile.\n\nSee the manual page for makegen for more information.\n"

typedef enum
{
	_NONE,
	_DEPS,
	_MAIN_LABEL,
	_MAIN_OFILE,
	_OFILES,
	_LIBS,
	_INSTALL_DIR
} _DEFAULT_STATES;

typedef struct
{
	char deps[MAXLINELEN];
	char main_label[MAXLINELEN];
	char main_ofile[64];
	char ofiles[MAXLINELEN];
	char libs[MAXLINELEN];
	char install_dir[64];
} makefile;

typedef struct
{
	char progname[MAXLINELEN];
	char version [MAXLINELEN];
	char generated[MAXLINELEN];
} header;

makefile _makefile;
header _header;

int exist(char file[]);

int blank(char file[]);

char * strsub(char string[], char find[], char replace[]);

void mktmpfiles(char ext[5]);

void mkotmpfile(void);

void set_deps(void);

void set_ofiles(void);

void create_makefile(void);

void delete_tmpfiles(void);

#endif /* _MAKEGEN_H */
